unit Amform;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs
  , Menus;

type
  (* * * * ARABIC FORM * * * *)
  TAForm = class(TComponent)
  private
    FArabic: Boolean;
    procedure SetArabic ( b:Boolean);
  public
    constructor Create(AOwner: TComponent); override;
  published
    property Arabic:Boolean read FArabic write SetArabic default true;
  end;
  (* * * * ARABIC MAINMENU * * * *)
  TAMainMenu = class(TMainMenu)
  private
    FArabic: Boolean;
    procedure SetArabic ( b:Boolean);
  protected
    function GetHandle:THandle; override;
  public
    constructor Create (Aowner:TComponent); override;
  published
    property Arabic:Boolean read FArabic write SetArabic default true;
  end;
  (* * * * ARABIC POPUPMENU * * * *)
  TAPopupMenu = class(TPopupMenu)
  private
    FArabic: Boolean;
  protected
    procedure Popup(x,y:integer); override;
  public
    constructor Create (Aowner:TComponent); override;
  published
    property Arabic:Boolean read FArabic write FArabic default true;
  end;


procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Rayed Comp', [TAMainMenu]);
  RegisterComponents('Rayed Comp', [TAPopupMenu]);
  RegisterComponents('Rayed Comp', [TAForm]);
end;

  (* * * * ARABIC FORM * * * *)
constructor TAForm.Create(AOwner: TComponent);
begin
 inherited Create(AOwner);
 Arabic:= True;
end;

procedure TAForm.SetArabic ( b:Boolean);
var
 L:longint;
begin
 FArabic:= b;
 L:= GetWindowLong ( (owner as TForm).handle, GWL_EXSTYLE );
 if FArabic
    then L:= L or WS_EX_RTLREADING or WS_EX_LEFTSCROLLBAR or WS_EX_RIGHT
    else L:= L and not ( WS_EX_RTLREADING or WS_EX_LEFTSCROLLBAR or WS_EX_RIGHT);
 SetWindowLong ( (owner as TForm).handle, GWL_EXSTYLE , L);
end;

  (* * * * ARABIC MAINMENU * * * *)
constructor TAMainMenu.Create(AOwner: TComponent);
begin
 inherited Create(AOwner);
 Arabic:= True;
end;

function TAMainMenu.GetHandle:THandle;
var
 mii: TMENUITEMINFO;
 szBuffer: array[0..80] of char;
begin
 Result:= inherited GetHandle;

 // Get the original value of mii.fType first
 // and OR that with MFT_RIGHTORDER
 mii.cbSize := sizeof (TMENUITEMINFO);
 mii.fMask := MIIM_TYPE;
 mii.dwTypeData:= szBuffer;
 mii.cch   := sizeof (szBuffer);

 GetMenuItemInfo(Result, 0, TRUE, mii);

 // OR in MFT_RIGHTORDER type
 mii.fMask := MIIM_TYPE;
 if FArabic
    then mii.fType  := mii.fType or MFT_RIGHTORDER or MFT_RIGHTJUSTIFY
    else mii.fType  := mii.fType and not (MFT_RIGHTORDER or MFT_RIGHTJUSTIFY) ;

 // Right order the menu
 SetMenuItemInfo(Result, 0, TRUE, mii);
 DrawMenuBar(WindowHandle);
end;


procedure TAMainMenu.SetArabic ( b:Boolean);
begin
 FArabic:= b;
 GetHandle;
end;

  (* * * * ARABIC POPUPMENU * * * *)
constructor TAPopupMenu.Create(AOwner: TComponent);
begin
 inherited Create(AOwner);
 Arabic:= True;
end;

procedure TAPopupMenu.Popup(x,y:integer);
var
 mii: TMENUITEMINFO;
 szBuffer: array[0..80] of char;
begin
 // Get the original value of mii.fType first
 // and OR that with MFT_RIGHTORDER
 mii.cbSize := sizeof (TMENUITEMINFO);
 mii.fMask := MIIM_TYPE;
 mii.dwTypeData:= szBuffer;
 mii.cch   := sizeof (szBuffer);

 GetMenuItemInfo(handle, 0, TRUE, mii);

 // OR in MFT_RIGHTORDER type
 mii.fMask := MIIM_TYPE;
 if FArabic
    then mii.fType  := mii.fType or MFT_RIGHTORDER or MFT_RIGHTJUSTIFY
    else mii.fType  := mii.fType and not (MFT_RIGHTORDER or MFT_RIGHTJUSTIFY) ;

 // Right order the menu
 SetMenuItemInfo(handle, 0, TRUE, mii);

 inherited Popup(x,y);

end;

end.
